/*
 * $Id: AnnounceRegist.cpp 711 2008-02-10 18:52:44Z sayer $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 * Copyright (C) 2006 iptego GmbH
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * For a license to use the sems software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "OpenJTalk.h"
#include "AmConfig.h"
#include "AmUtils.h"

#include "sems.h"
#include "log.h"

#include "ampi/UACAuthAPI.h"
#include "ampi/SIPRegistrarClientAPI.h"
#include "AmUAC.h"
#include "AmPlugIn.h"

#include "open_jtalk.h"

#define MOD_NAME "open_jtalk"

EXPORT_SESSION_FACTORY(AnnounceRegistFactory,MOD_NAME);

string AnnounceRegistFactory::AnnouncePath;
string AnnounceRegistFactory::AnnounceFile;

AnnounceRegistFactory::AnnounceRegistFactory(const string& _app_name)
  : AmSessionFactory(_app_name)
{
}

int AnnounceRegistFactory::onLoad()
{
    AmConfigReader cfg;
    if(cfg.loadFile(AmConfig::ModConfigPath + string(MOD_NAME ".conf")))
	return -1;

    // get application specific global parameters
    configureModule(cfg);

    AnnouncePath = cfg.getParameter("announce_path",ANNOUNCE_PATH);
    if( !AnnouncePath.empty() 
	&& AnnouncePath[AnnouncePath.length()-1] != '/' )
	AnnouncePath += "/";

    AnnounceFile = cfg.getParameter("default_announce",ANNOUNCE_FILE);

    string announce_file = AnnouncePath + AnnounceFile;
#if 0
    if(!file_exists(announce_file)){
	ERROR("default file for ann_b2b module does not exist ('%s').\n",
	      announce_file.c_str());
	return -1;
    }
#endif

    OpenJTalkInfo ojti;
    ojti.dn_mecab = cfg.getParameter("dn_mecab","");
    ERROR("dn_mecab = %s\n", ojti.dn_mecab.c_str());
    ojti.fn_ms_lf0 = cfg.getParameter("fn_ms_lf0","");
    ojti.fn_ms_mcp = cfg.getParameter("fn_ms_mcp","");
    ojti.fn_ms_dur = cfg.getParameter("fn_ms_dur","");
    ojti.fn_ts_lf0 = cfg.getParameter("fn_ts_lf0","");
    ojti.fn_ts_mcp = cfg.getParameter("fn_ts_mcp","");
    ojti.fn_ts_dur = cfg.getParameter("fn_ts_dur","");
    int num, i;
    char optname[32];
    num = cfg.getParameterInt("num_ws_lf0",0);
    for(i = 1; i <= num; ++i) {
      sprintf(optname, "fn_ws_lf0_%d", i);
      ojti.fn_ws_lf0.push_back(cfg.getParameter(optname,""));
    }
    num = cfg.getParameterInt("num_ws_mcp",0);
    for(i = 1; i <= num; ++i) {
      sprintf(optname, "fn_ws_mcp_%d", i);
      ojti.fn_ws_mcp.push_back(cfg.getParameter(optname,""));
    }
    ojti.fn_ms_gvl = cfg.getParameter("fn_ms_gvl","");
    ojti.fn_ms_gvm = cfg.getParameter("fn_ms_gvm","");
    ojti.fn_ts_gvl = cfg.getParameter("fn_ts_gvl","");
    ojti.fn_ts_gvm = cfg.getParameter("fn_ts_gvm","");
    ojti.fn_gv_switch = cfg.getParameter("fn_gv_switch","");

    /* global parameter */
    int sampling_rate = 16000;
    int fperiod = 80;
    double alpha = 0.42;
    int stage = 0;               /* gamma = -1.0/stage */
    double beta = 0.0;
    int audio_buff_size = 1600;
    double uv_threshold = 0.5;
    double gv_weight_lf0 = 0.7;
    double gv_weight_mcp = 1.0;
    HTS_Boolean use_log_gain = FALSE;

    /* engine */
    OpenJTalk open_jtalk;

    /* initialize and load */
    char **fn_ws_lf0;
    char **fn_ws_mcp;
    fn_ws_lf0 = (char **) calloc(ojti.fn_ws_lf0.size(), sizeof(char *));
    list<string>::iterator pos;
    i = 0;
    for(pos=ojti.fn_ws_lf0.begin(); pos!=ojti.fn_ws_lf0.end(); ++pos){
      fn_ws_lf0[i] = (char *)(*pos).c_str();
//      ERROR("fn_ws_lf0[%d] = %s\n", i, fn_ws_lf0[i]);
      ++i;
    }
    fn_ws_mcp = (char **) calloc(ojti.fn_ws_mcp.size(), sizeof(char *));
    i = 0;
    for(pos=ojti.fn_ws_mcp.begin(); pos!=ojti.fn_ws_mcp.end(); ++pos){
      fn_ws_mcp[i] = (char *)(*pos).c_str();
//      ERROR("fn_ws_mcp[%d] = %s\n", i, fn_ws_mcp[i]);
      ++i;
    }

    OpenJTalk_initialize(&open_jtalk, sampling_rate, fperiod, alpha,
      stage, beta, audio_buff_size, uv_threshold, use_log_gain, 
      gv_weight_mcp, gv_weight_lf0);

#if 0
    ERROR("%s : %s : %s : %s : %s : %s : %s : %s : %s : %s : %s : %s\n", (char *)ojti.dn_mecab.c_str(),
       (char *)ojti.fn_ms_dur.c_str(), (char *)ojti.fn_ts_dur.c_str(),
       (char *)ojti.fn_ms_mcp.c_str(), (char *)ojti.fn_ts_mcp.c_str(),
//       fn_ws_mcp, ojti.fn_ws_mcp.size(),
       (char *)ojti.fn_ms_lf0.c_str(), (char *)ojti.fn_ts_lf0.c_str(),
//       fn_ws_lf0, ojti.fn_ws_lf0.size(),
       (char *)ojti.fn_ms_gvm.c_str(),
       (char *)ojti.fn_ts_gvm.c_str(), (char *)ojti.fn_ms_gvl.c_str(),
       (char *)ojti.fn_ts_gvl.c_str(), (char *)ojti.fn_gv_switch.c_str());
#endif

    OpenJTalk_load(&open_jtalk, (char *)ojti.dn_mecab.c_str(),
       (char *)ojti.fn_ms_dur.c_str(), (char *)ojti.fn_ts_dur.c_str(),
       (char *)ojti.fn_ms_mcp.c_str(), (char *)ojti.fn_ts_mcp.c_str(),
       fn_ws_mcp, ojti.fn_ws_mcp.size(),
       (char *)ojti.fn_ms_lf0.c_str(), (char *)ojti.fn_ts_lf0.c_str(),
       fn_ws_lf0, ojti.fn_ws_lf0.size(),
       (char *)ojti.fn_ms_gvm.c_str(),
       (char *)ojti.fn_ts_gvm.c_str(), (char *)ojti.fn_ms_gvl.c_str(),
       (char *)ojti.fn_ts_gvl.c_str(), (char *)ojti.fn_gv_switch.c_str());

    FILE *wavfp = NULL;
    wavfp = fopen(announce_file.c_str(), "wb");
    string sample_text = cfg.getParameter("sample_text","");
    ERROR("sample_text = %s\n", sample_text.c_str());

    /* synthesis */
    OpenJTalk_synthesis(&open_jtalk, (char *)sample_text.c_str(), wavfp, NULL);

    /* free */
    OpenJTalk_clear(&open_jtalk);
    free(fn_ws_mcp);
    free(fn_ws_lf0);
    if (wavfp != NULL)
      fclose(wavfp);

    // stay backwards compatible
    RegInfo ri; 
    ri.domain = cfg.getParameter("domain","");
    ri.user = cfg.getParameter("user","");
    ri.display_name = cfg.getParameter("display_name","");
    ri.auth_user = cfg.getParameter("auth_user","");
    ri.passwd = cfg.getParameter("pwd","");

    dialer.add_reg(ri);
    dialer.start();

    return 0;
}

AmSession* AnnounceRegistFactory::onInvite(const AmSipRequest& req)
{
    string announce_file = AnnouncePath + AnnounceFile;
    
    AnnounceRegistDialog* dlg = new AnnounceRegistDialog(announce_file);
    
   return dlg;
}

void AnnounceRegistFactory::postEvent(AmEvent* ev) {
    dialer.postEvent(ev);
}

//
// AnnounceRegistDialog class
//

AnnounceRegistDialog::AnnounceRegistDialog(const string& filename)
    : filename(filename) 
{

}

AnnounceRegistDialog::~AnnounceRegistDialog()
{
}

void AnnounceRegistDialog::onSessionStart(const AmSipRequest& req)
{
    DBG("AnnounceRegistDialog::onSessionStart\n");
    startSession();
}

void AnnounceRegistDialog::onSessionStart(const AmSipReply& rep)
{
    DBG("AnnounceRegistDialog::onSessionStart (SEMS originator mode)\n");
    startSession();
}

void AnnounceRegistDialog::startSession(){
    // disable DTMF detection - don't use DTMF here
    setDtmfDetectionEnabled(false);

    if(wav_file.open(filename,AmAudioFile::Read))
	throw string("AnnounceRegistDialog::onSessionStart: Cannot open file\n");
    
    setOutput(&wav_file);
}

void AnnounceRegistDialog::onBye(const AmSipRequest& req)
{
    DBG("onBye: stopSession\n");
    setStopped();
}

void AnnounceRegistDialog::process(AmEvent* event)
{

    AmAudioEvent* audio_event = dynamic_cast<AmAudioEvent*>(event);
    if(audio_event && (audio_event->event_id == AmAudioEvent::cleared)){
	dlg.bye();
	setStopped();
	return;
    }

    AmSession::process(event);
}

//
// RegThread class
//

void RegThread::add_reg(const RegInfo& ri) {
  registrations.push_back(ri);
}

void RegThread::create_registration(RegInfo& ri) {
  AmDynInvokeFactory* di_f = AmPlugIn::instance()->getFactory4Di("registrar_client");
  if (di_f == NULL) {
    ERROR("unable to get a registrar_client\n");
  } else {
    AmDynInvoke* uac_auth_i = di_f->getInstance();
    if (uac_auth_i!=NULL) {

      DBG("calling createRegistration\n");
      AmArg di_args, reg_handle;
      di_args.push(ri.domain.c_str());
      di_args.push(ri.user.c_str());
      di_args.push(ri.display_name.c_str()); // display name
      di_args.push(ri.auth_user.c_str());  // auth_user
      di_args.push(ri.passwd.c_str());    // pwd
      di_args.push(MOD_NAME); //sess_link
                        
      uac_auth_i->invoke("createRegistration", di_args, reg_handle);
      if (reg_handle.size()) 
        ri.handle = reg_handle.get(0).asCStr();
    }
  }
}

bool RegThread::check_registration(const RegInfo& ri) {
  if (!ri.handle.length())
    return false;
  AmDynInvokeFactory* di_f = AmPlugIn::instance()->getFactory4Di("registrar_client");
  if (di_f == NULL) {
    ERROR("unable to get a registrar_client\n");
  } else {
    AmDynInvoke* uac_auth_i = di_f->getInstance();
    if (uac_auth_i!=NULL) {

      AmArg di_args, res;
      di_args.push(ri.handle.c_str());
      uac_auth_i->invoke("getRegistrationState", di_args, res);
      if (res.size()) {
        if (!res.get(0).asInt())
          return false; // does not exist
        int state = res.get(1).asInt();
        int expires = res.get(2).asInt();
        DBG("Got state %s with expires %us for registration.\n", 
            getSIPRegistationStateString(state), expires);
        if (state == 2) // expired ... FIXME: add values from API here
          return false;
        // else pending or active
        return true;
      }
    }
  }
  return false;
}

void RegThread::run() {
  DBG("registrar client started.\n");
  sleep(2); // wait for sems to completely start up

  while (true) {
    for (vector<RegInfo>::iterator it = registrations.begin(); 
         it != registrations.end(); it++) {
      if (!check_registration(*it)) {
        DBG("Registration %d does not exist or timeout. Creating registration.\n",
            (int)(it - registrations.begin()));
        create_registration(*it);
      }
    }
    sleep(10); // 10 seconds
  }
                
}

void RegThread::on_stop() {
  DBG("not stopping...\n");
}

void RegThread::postEvent(AmEvent* ev) {
  DBG("received registration event.\n"); 
  // TODO: handle events instead of re-try
  delete ev;
}
